/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.chain.impl.spec;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;

import fr.obeo.acceleo.bridge.ui.AcceleoBridgeUiPlugin;
import fr.obeo.acceleo.bridge.ui.popupMenus.AcceleoConvertMof2EmfOperation;
import fr.obeo.acceleo.chain.AcceleoChainMessages;
import fr.obeo.acceleo.chain.ModelSet;
import fr.obeo.acceleo.chain.impl.ConvertXmiImpl;
import fr.obeo.acceleo.chain.tools.CObject;
import fr.obeo.acceleo.chain.tools.ChainLog;
import fr.obeo.acceleo.gen.IGenFilter;
import fr.obeo.acceleo.gen.template.eval.LaunchManager;
import fr.obeo.acceleo.tools.resources.Resources;

/**
 * A representation of the model object '<em><b>ConvertXmi</b></em>'. It
 * converts a xmi from MOF to EMF.
 * 
 * @author www.obeo.fr
 * 
 */
public class CConvertXmi extends ConvertXmiImpl implements CObject {

	/* (non-Javadoc) */
	public void launch(CChain cchain, IGenFilter genFilter, IProgressMonitor monitor, LaunchManager mode) throws CoreException {
		if (mode.getMode() != LaunchManager.PREVIEW_MODE && mode.getMode() != LaunchManager.PHANTOM_MODE) {
			StringBuffer report = new StringBuffer(""); //$NON-NLS-1$
			IContainer container = cchain.getFile().getProject().getWorkspace().getRoot();
			List mofList = new ArrayList();
			if (getMof() instanceof ModelSet) {
				IContainer mofSet = Resources.getContainer(container, new Path(getMof().getPath()), monitor);
				if (mofSet != null && mofSet.exists()) {
					IFile[] files = Resources.members(mofSet);
					for (int i = 0; i < files.length; i++) {
						IFile file = files[i];
						mofList.add(file.getFullPath().toString());
					}
				} else {
					report.append(AcceleoChainMessages.getString("MissingFolder", new Object[] { getMof().getPath(), })); //$NON-NLS-1$
				}
			} else if (getMof() != null) {
				mofList.add(getMof().getPath());
			}
			Iterator mofIt = mofList.iterator();
			while (mofIt.hasNext()) {
				String mofPath = (String) mofIt.next();
				IFile mof = container.getFile(new Path(mofPath));
				if (!mof.exists()) {
					report.append(AcceleoChainMessages.getString("CConvertXmi.UnresolvedMofModel", new Object[] { mofPath, })); //$NON-NLS-1$
				} else if (getEmf().getPath().trim().length() == 0) {
					report.append(AcceleoChainMessages.getString("CConvertXmi.InvalidEMFModel")); //$NON-NLS-1$
				} else {
					String emfPath;
					if (getEmf() instanceof ModelSet) {
						emfPath = new Path(getEmf().getPath()).append(mof.getName()).toString();
					} else {
						emfPath = getEmf().getPath();
					}
					try {
						AcceleoConvertMof2EmfOperation operation = new AcceleoConvertMof2EmfOperation(mof, emfPath);
						operation.run(monitor);
						IFile generated = Resources.findFile(new Path(emfPath));
						if (generated != null) {
							List files = new ArrayList();
							files.add(generated);
							cchain.addGeneratedFiles(files);
						}
					} catch (Exception e) {
						AcceleoBridgeUiPlugin.getDefault().log(e, true);
					}
				}
			}
			ChainLog.report(cchain, this, report, monitor, mode);
		}
	}

	/* (non-Javadoc) */
	public int totalWork() {
		return 0;
	}

}
