/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.tools.resources;

import org.eclipse.compare.rangedifferencer.IRangeComparator;

/**
 * The token comparator used by Acceleo to compare files.
 * 
 * @author www.obeo.fr
 * 
 */
public class AcceleoTokenComparator extends org.eclipse.compare.internal.TokenComparator {

	private String fText;

	/**
	 * Constructor.
	 * 
	 * @param text
	 *            is the text
	 * @param shouldEscape
	 *            indicates if we should escape
	 */
	public AcceleoTokenComparator(String text, boolean shouldEscape) {
		super(text, shouldEscape);
		fText = text;
	}

	/**
	 * Constructor.
	 * 
	 * @param text
	 *            is the text
	 */
	public AcceleoTokenComparator(String text) {
		super(text);
		fText = text;
	}

	/* (non-Javadoc) */
	public boolean rangesEqual(int thisIndex, IRangeComparator other, int otherIndex) {
		if (other != null && getClass() == other.getClass()) {
			AcceleoTokenComparator tc = (AcceleoTokenComparator) other;
			int thisStart = getTokenStart(thisIndex);
			int thisLen = getTokenLength(thisIndex);
			int otherStart = tc.getTokenStart(otherIndex);
			int otherLen = tc.getTokenLength(otherIndex);
			String thisToken = fText.substring(thisStart, thisStart + thisLen);
			String otherToken = tc.fText.substring(otherStart, otherStart + otherLen);
			if (thisToken.equals(otherToken)) {
				return true;
			} else if (thisToken.trim().length() == 0 && otherToken.trim().length() == 0) {
				return true;
			} else if (thisLen == otherLen) {
				return fText.regionMatches(false, getTokenStart(thisIndex), tc.fText, tc.getTokenStart(otherIndex), thisLen);
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

}
